package SomeType;

use Moose;

has 'color' => (is => 'rw');
has 'model' => (is => 'rw'); 
has 'mileage' => (is => 'rw');

"one, but we're not the same";

__END__


=head1 NAME

SomeType - Very simple class for testing Hessian class serialization.

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 INTERFACE


